
#include "model_common.h"

#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include "access.h"
#include "access_config.h"
#include "nrf_mesh_assert.h"
#include "nrf_mesh_utils.h"
#include "nordic_common.h"
#include "app_onoff.h"
#include "log.h"
#include "generic_onoff_server.h"
#include "generic_onoff_messages.h"
#include "battery_client_model.h"

#define GENERIC_ONOFF_OPCODE_SET22  0x00C1

/*battery model*/
#define BLE_UUID_BATTERY_MODEL_ID   0x100D //180F

//uint32_t battery_double_init(generic_onoff_server_t * p_server, uint8_t element_index);

static void battery_status_handle(access_model_handle_t handle, const access_message_rx_t * p_rx_msg, void * p_args)
{
    generic_battery_client_t * p_client = (generic_battery_client_t *) p_args;
    generic_battery_status_params_t in_data = {0};

    if (p_rx_msg->length == GENERIC_ONOFF_STATUS_MINLEN || p_rx_msg->length == GENERIC_ONOFF_STATUS_MAXLEN)
    {
        generic_battery_status_msg_pkt_t * p_msg_params_packed = (generic_battery_status_msg_pkt_t *) p_rx_msg->p_data;

        if (p_rx_msg->length == GENERIC_ONOFF_STATUS_MINLEN)
        {
            in_data.battery_State = p_msg_params_packed->battery_State;
            in_data.remaining_time_ms = 0;
        }
        else
        {
            in_data.battery_State = p_msg_params_packed->battery_State;
            in_data.remaining_time_ms = model_transition_time_decode(p_msg_params_packed->remaining_time);
        }

        p_client->settings.p_callbacks->battery_status_cb(p_client, &p_rx_msg->meta_data, &in_data);
    }
}

/*
uint32_t battery_init(app_onoff_server_t * p_battery, uint8_t element_index)
{   
    uint32_t status;
    status = battery_double_init(&p_battery->server, element_index);
    if(status |= NRF_SUCCESS)
    {
        __LOG(LOG_SRC_APP, LOG_LEVEL_INFO, "Battery init error\n");
    }
    return status;
}
*/

static const access_opcode_handler_t m_opcode_handlers2[] =
{
    {ACCESS_OPCODE_SIG(GENERIC_ONOFF_OPCODE_SET22), battery_status_handle},
};

uint32_t battery_double_init(generic_battery_client_t * p_client, uint8_t element_index)
{
    if (p_client == NULL ||
        p_client->settings.p_callbacks == NULL ||
        p_client->settings.p_callbacks->battery_status_cb == NULL ||
        p_client->settings.p_callbacks->periodic_publish_cb == NULL)
    {
        return NRF_ERROR_NULL;
    }

    if (p_client->settings.timeout == 0)
    {
        p_client->settings.timeout= MODEL_ACKNOWLEDGED_TRANSACTION_TIMEOUT;
    }

    access_model_add_params_t init_params =
    {
        .model_id = ACCESS_MODEL_SIG(BLE_UUID_BATTERY_MODEL_ID),
        .element_index =  0,
        .p_opcode_handlers = &m_opcode_handlers2[0],
        .opcode_count = ARRAY_SIZE(m_opcode_handlers2),
        .p_args = p_client,
        .publish_timeout_cb = NULL
    };

    uint32_t status = access_model_add(&init_params, &p_client->model_handle);

    if (status == NRF_SUCCESS)
    {
        status = access_model_subscription_list_alloc(p_client->model_handle);
    }
    return status;
}  